@@ -24,10 +24,6 @@
 #include <sys/stat.h>
 #include "dir_mutex.h"
 
-#ifdef _WIN32
-#define mkdir(a,b) mkdir(a)
-#endif
-
 int mkdirat(int dirfd, const char *pathname, mode_t mode)
 {
 	int rc;
