@@ -345,6 +345,27 @@ typedef NTSTATUS (WINAPI *NtQueryDirectoryFile_t)(
   PUNICODE_STRING FileName,
   BOOLEAN RestartScan);
 
+typedef NTSTATUS (WINAPI *NtQueryDirectoryFileEx_t)(
+  HANDLE FileHandle,
+  HANDLE Event,
+  PIO_APC_ROUTINE ApcRoutine,
+  PVOID ApcContext,
+  PIO_STATUS_BLOCK IoStatusBlock,
+  PVOID FileInformation,
+  ULONG Length,
+  FILE_INFORMATION_CLASS FileInformationClass,
+  ULONG QueryFlags,
+  PUNICODE_STRING FileName
+  );
+
+typedef NTSTATUS (WINAPI *NtSetInformationFile_t)(
+  HANDLE                 FileHandle,
+  PIO_STATUS_BLOCK       IoStatusBlock,
+  PVOID                  FileInformation,
+  ULONG                  Length,
+  FILE_INFORMATION_CLASS FileInformationClass
+);
+
 typedef int (*access_t)(const char *pathname, int mode);
 typedef int (*rename_t)(const char *oldpath, const char *newpath);
 
@@ -387,6 +408,8 @@ static NtCreateFile_t			NtCreateFile_orig;
 static NtOpenFile_t			NtOpenFile_orig;
 static NtCreateUserProcess_t		NtCreateUserProcess_orig;
 static NtQueryDirectoryFile_t		NtQueryDirectoryFile_orig;
+static NtQueryDirectoryFileEx_t		NtQueryDirectoryFileEx_orig;
+static NtSetInformationFile_t		NtSetInformationFile_orig;
 static access_t				_access_orig;
 static rename_t				rename_orig;
 
@@ -634,6 +657,123 @@ out_exit:
 	return rc;
 }
 
+NTSTATUS WINAPI NtQueryDirectoryFileEx_hook(
+  HANDLE FileHandle,
+  HANDLE Event,
+  PIO_APC_ROUTINE ApcRoutine,
+  PVOID ApcContext,
+  PIO_STATUS_BLOCK IoStatusBlock,
+  PVOID FileInformation,
+  ULONG Length,
+  FILE_INFORMATION_CLASS FileInformationClass,
+  ULONG QueryFlags,
+  PUNICODE_STRING FileName)
+{
+	wchar_t widepath[WIDE_PATH_MAX];
+	NTSTATUS rc;
+	DWORD len;
+	if(FileName) {
+		DEBUG_HOOK("NtQueryDirectoryFileEx: %.*ls\n", FileName->Length/2, FileName->Buffer);
+	} else {
+		DEBUG_HOOK("NtQueryDirectoryFileEx: (No Filename)\n");
+	}
+
+	rc = NtQueryDirectoryFileEx_orig(FileHandle,
+					 Event,
+					 ApcRoutine,
+					 ApcContext,
+					 IoStatusBlock,
+					 FileInformation,
+					 Length,
+					 FileInformationClass,
+					 QueryFlags,
+					 FileName);
+	if(!FileName) {
+		/* There's no FileName if we're doing the equivalent of
+		 * readdir(), and in that case we already had to CreateFile()
+		 * on the directory name in order to get here, so just return.
+		 */
+		return rc;
+	}
+	DWORD save_error = GetLastError();
+
+	len = GetFinalPathNameByHandleW(FileHandle, widepath, WIDE_PATH_MAX, FILE_NAME_NORMALIZED);
+	if(len == 0) {
+		/* Failed to get path for some reason */
+		DEBUG_HOOK("NtQueryDirectoryFileEx Error - failed to GetFinalPathNameByHandle\n");
+		goto out_exit;
+	}
+	if(len + 1 + FileName->Length/2 + 1 > WIDE_PATH_MAX) {
+		/* Path too large. */
+		DEBUG_HOOK("NtQueryDirectorFileEx Error - path too long (%i, %i)\n", len, FileName->Length);
+		goto out_exit;
+	}
+	swprintf(widepath+len, WIDE_PATH_MAX-len, L"\\%.*ls", FileName->Length/2, FileName->Buffer);
+	DEBUG_HOOK(" - got full NtQueryDirectoryFileEx path: '%ls'\n", widepath);
+	handle_file_w(widepath, -1, NULL, ACCESS_READ);
+
+out_exit:
+	SetLastError(save_error);
+	return rc;
+}
+
+NTSTATUS WINAPI NtSetInformationFile_hook(
+  HANDLE                 FileHandle,
+  PIO_STATUS_BLOCK       IoStatusBlock,
+  PVOID                  FileInformation,
+  ULONG                  Length,
+  FILE_INFORMATION_CLASS FileInformationClass
+)
+{
+	wchar_t widepath[WIDE_PATH_MAX];
+	wchar_t destpath[WIDE_PATH_MAX];
+	NTSTATUS rc;
+	DWORD len;
+	FILE_RENAME_INFORMATION *info = FileInformation;
+	int failed = 0;
+
+	if(FileInformationClass == FileRenameInformation) {
+		len = GetFinalPathNameByHandleW(FileHandle, widepath, WIDE_PATH_MAX, FILE_NAME_NORMALIZED);
+		if(len == 0) {
+			/* Failed to get path for some reason */
+			DEBUG_HOOK("NtSetInformationFile Error - failed to GetFinalPathNameByHandle\n");
+			failed = 1;
+		}
+		if(len + 1 + info->FileNameLength/2 + 1 > WIDE_PATH_MAX) {
+			/* Path too large. */
+			DEBUG_HOOK("NtSetInformationFile Error - path too long (%i, %i)\n", len, info->FileNameLength);
+			failed = 1;
+		}
+	}
+
+	rc = NtSetInformationFile_orig(FileHandle,
+				       IoStatusBlock,
+				       FileInformation,
+				       Length,
+				       FileInformationClass);
+	if(rc != STATUS_SUCCESS || failed)
+		return rc;
+
+	/* We're only checking this to see if a file gets renamed via this call. */
+	if(FileInformationClass != FileRenameInformation)
+		return rc;
+
+	if(info->FileNameLength / 2 >= WIDE_PATH_MAX) {
+		DEBUG_HOOK("NtSetInformationFile error - new path is too long.\n");
+		return rc;
+	}
+	DWORD save_error = GetLastError();
+
+	wcsncpy(destpath, info->FileName, info->FileNameLength);
+	destpath[info->FileNameLength / 2] = 0;
+
+	DEBUG_HOOK("NtSetInformationFile: rename '%S' -> '%S'\n", widepath, destpath);
+	handle_file_w(widepath, -1, destpath, ACCESS_RENAME);
+
+	SetLastError(save_error);
+	return rc;
+}
+
 BOOL WINAPI DeleteFileA_hook(
     __in LPCSTR lpFileName)
 {
@@ -1368,6 +1508,8 @@ static struct patch_entry patch_table[] = {
 	HOOK(NtOpenFile),
 	HOOK(NtCreateUserProcess),
 	HOOK(NtQueryDirectoryFile),
+	HOOK(NtQueryDirectoryFileEx),
+	HOOK(NtSetInformationFile),
 #undef MODULE_NAME
 #define MODULE_NAME "msvcrt.dll"
 	HOOK(_access),
@@ -1384,8 +1526,7 @@ static char execdir[MAX_PATH];
 
 void tup_inject_setexecdir(const char* dir)
 {
-	execdir[0] = '\0';
-	strncat(execdir, dir, MAX_PATH);
+	strncpy(execdir, dir, MAX_PATH - 1);
 	execdir[MAX_PATH - 1] = '\0';
 }
 
@@ -1476,6 +1617,7 @@ static int canon_path(const wchar_t *file, int filelen, char *dest)
 	int len;
 	int count;
 	int branch;
+	int x;
 	if(!file) {
 		DEBUG_HOOK("canon_path: No file - return 0\n");
 		goto out_empty;
@@ -1570,6 +1712,11 @@ static int canon_path(const wchar_t *file, int filelen, char *dest)
 	if(!count) {
 		goto out_empty;
 	}
+	/* Use forward-slashes inside tup, for things like regex matching. */
+	for(x=0; x<count; x++) {
+		if(dest[x] == '\\')
+			dest[x] = '/';
+	}
 	DEBUG_HOOK("WideCharToMultiByte[%ls] -> %i, '%s'\n", widefullpath, count, dest);
 
 	/* Discount the nul-terminator */
