@@ -33,6 +33,7 @@
 #define _GNU_SOURCE
 #include "tup/access_event.h"
 #include "tup/flock.h"
+#include "tup/ccache.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -73,6 +74,7 @@ static int (*s_rename)(const char*, const char*);
 static int (*s_renameat)(int, const char*, int, const char*);
 static int (*s_mkstemp)(char *template);
 static int (*s_mkostemp)(char *template, int flags);
+static int (*s_remove)(const char *);
 static int (*s_unlink)(const char*);
 static int (*s_unlinkat)(int, const char*, int);
 static int (*s_execve)(const char *filename, char *const argv[],
@@ -110,10 +112,39 @@ static int errored = 0;
 static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
 static int depfd = -1;
 
+static void prepare(void)
+{
+	pthread_mutex_lock(&mutex);
+}
+
+static void parent(void)
+{
+	int rc;
+	rc = pthread_mutex_unlock(&mutex);
+	if(rc != 0) {
+		fprintf(stderr, "tup error: pthread_mutex_unlock() failed in parent atfork handler with rc=%i\n", rc);
+		errored = 1;
+	}
+}
+
+static void child(void)
+{
+	int rc;
+	rc = pthread_mutex_unlock(&mutex);
+	if(rc != 0) {
+		fprintf(stderr, "tup error: pthread_mutex_unlock() failed in child atfork handler with rc=%i\n", rc);
+		errored = 1;
+	}
+}
+
 static void init_fd(void) __attribute__((constructor));
 static void init_fd(void)
 {
 	const char *depfile;
+	if(pthread_atfork(prepare, parent, child) != 0) {
+		fprintf(stderr, "tup error: Unable to set pthread atfork handlers.\n");
+		goto out_error;
+	}
 	depfile = getenv(TUP_DEPFILE);
 	if(!depfile) {
 		fprintf(stderr, "tup error: Unable to find dependency filename in the TUP_DEPFILE environment variable.\n");
@@ -154,8 +185,7 @@ int open(const char *pathname, int flags, ...)
 			at = ACCESS_WRITE;
 		handle_file(pathname, "", at);
 	} else {
-		if(errno == ENOENT || errno == ENOTDIR)
-			handle_file(pathname, "", ACCESS_READ);
+		handle_file(pathname, "", ACCESS_READ);
 	}
 	return rc;
 }
@@ -180,8 +210,7 @@ int open64(const char *pathname, int flags, ...)
 			at = ACCESS_WRITE;
 		handle_file(pathname, "", at);
 	} else {
-		if(errno == ENOENT || errno == ENOTDIR)
-			handle_file(pathname, "", ACCESS_READ);
+		handle_file(pathname, "", ACCESS_READ);
 	}
 	return rc;
 }
@@ -195,8 +224,7 @@ FILE *fopen(const char *path, const char *mode)
 	if(f) {
 		handle_file(path, "", !(mode[0] == 'r'));
 	} else {
-		if(errno == ENOENT || errno == ENOTDIR)
-			handle_file(path, "", ACCESS_READ);
+		handle_file(path, "", ACCESS_READ);
 	}
 	return f;
 }
@@ -210,8 +238,7 @@ FILE *fopen64(const char *path, const char *mode)
 	if(f) {
 		handle_file(path, "", !(mode[0] == 'r'));
 	} else {
-		if(errno == ENOENT || errno == ENOTDIR)
-			handle_file(path, "", ACCESS_READ);
+		handle_file(path, "", ACCESS_READ);
 	}
 	return f;
 }
@@ -225,8 +252,7 @@ FILE *freopen(const char *path, const char *mode, FILE *stream)
 	if(f) {
 		handle_file(path, "", !(mode[0] == 'r'));
 	} else {
-		if(errno == ENOENT || errno == ENOTDIR)
-			handle_file(path, "", ACCESS_READ);
+		handle_file(path, "", ACCESS_READ);
 	}
 	return f;
 }
@@ -311,9 +337,7 @@ int rename(const char *old, const char *new)
 	WRAP(s_rename, "rename");
 	rc = s_rename(old, new);
 	if(rc == 0) {
-		if(!ignore_file(old) && !ignore_file(new)) {
-			handle_file(old, new, ACCESS_RENAME);
-		}
+		handle_file(old, new, ACCESS_RENAME);
 	}
 	return rc;
 }
@@ -333,9 +357,7 @@ int renameat(int oldfd, const char *old, int newfd, const char *new)
 	WRAP(s_renameat, "renameat");
 	rc = s_renameat(oldfd, old, newfd, new);
 	if(rc == 0) {
-		if(!ignore_file(old) && !ignore_file(new)) {
-			handle_file(old, new, ACCESS_RENAME);
-		}
+		handle_file(old, new, ACCESS_RENAME);
 	}
 	return rc;
 }
@@ -364,6 +386,17 @@ int mkostemp(char *template, int flags)
 	return rc;
 }
 
+int remove(const char *pathname)
+{
+	int rc;
+
+	WRAP(s_remove, "remove");
+	rc = s_remove(pathname);
+	if(rc == 0)
+		handle_file(pathname, "", ACCESS_UNLINK);
+	return rc;
+}
+
 int unlink(const char *pathname)
 {
 	int rc;
@@ -480,11 +513,7 @@ int __xstat(int vers, const char *name, struct stat *buf)
 	int rc;
 	WRAP(s_xstat, "__xstat");
 	rc = s_xstat(vers, name, buf);
-	if(rc < 0) {
-		if(errno == ENOENT || errno == ENOTDIR) {
-			handle_file(name, "", ACCESS_READ);
-		}
-	}
+	handle_file(name, "", ACCESS_READ);
 	return rc;
 }
 
@@ -493,11 +522,7 @@ int stat64(const char *filename, struct stat64 *buf)
 	int rc;
 	WRAP(s_stat64, "stat64");
 	rc = s_stat64(filename, buf);
-	if(rc < 0) {
-		if(errno == ENOENT || errno == ENOTDIR) {
-			handle_file(filename, "", ACCESS_READ);
-		}
-	}
+	handle_file(filename, "", ACCESS_READ);
 	return rc;
 }
 
@@ -507,11 +532,7 @@ int __xstat64(int __ver, __const char *__filename,
 	int rc;
 	WRAP(s_xstat64, "__xstat64");
 	rc = s_xstat64(__ver, __filename, __stat_buf);
-	if(rc < 0) {
-		if(errno == ENOENT || errno == ENOTDIR) {
-			handle_file(__filename, "", ACCESS_READ);
-		}
-	}
+	handle_file(__filename, "", ACCESS_READ);
 	return rc;
 }
 
@@ -520,11 +541,7 @@ int __lxstat64(int vers, const char *path, struct stat64 *buf)
 	int rc;
 	WRAP(s_lxstat64, "__lxstat64");
 	rc = s_lxstat64(vers, path, buf);
-	if(rc < 0) {
-		if(errno == ENOENT || errno == ENOTDIR) {
-			handle_file(path, "", ACCESS_READ);
-		}
-	}
+	handle_file(path, "", ACCESS_READ);
 	return rc;
 }
 
@@ -548,6 +565,8 @@ static void handle_file_locked(const char *dirname, int dirlen, const char *file
 		return;
 	if(ignore_file(file))
 		return;
+	if(ignore_file(file2))
+		return;
 
 	if(tup_flock(depfd) < 0) {
 		fprintf(stderr, "tup error: Unable to lock dependency file for writing [%i]: (file event = %s)\n", depfd, file);
@@ -630,6 +649,8 @@ static int ignore_file(const char *file)
 		return 1;
 	if(strncmp(file, "/proc/", 6) == 0)
 		return 1;
+	if(is_ccache_path(file))
+		return 1;
 	return 0;
 }
 
