@@ -32,6 +32,7 @@ struct path_element;
 struct pel_group;
 
 #define SOTGV_CREATE_DIRS 2
+#define SOTGV_IGNORE_DIRS 3
 
 int create_name_file(tupid_t dt, const char *file, time_t mtime,
 		     struct tup_entry **entry);
@@ -48,7 +49,7 @@ struct tup_entry *get_tent_dt(tupid_t dt, const char *path);
 tupid_t find_dir_tupid(const char *dir);
 tupid_t find_dir_tupid_dt(tupid_t dt, const char *dir,
 			  struct path_element **last, int sotgv, int full_deps);
-tupid_t find_dir_tupid_dt_pg(FILE *f, tupid_t dt, struct pel_group *pg,
+tupid_t find_dir_tupid_dt_pg(tupid_t dt, struct pel_group *pg,
 			     struct path_element **last, int sotgv, int full_deps);
 int gimme_tent(const char *name, struct tup_entry **entry);
 
