@@ -293,6 +293,32 @@ static struct node *find_or_create_node(struct graph *g, struct tup_entry *tent)
 	return n;
 }
 
+/* Callback for adding group dependencies to the regular update graph. This
+ * just adds a link from <group1> -> <group2> so that if we 'tup upd
+ * <group2>', we also build everything in group1. See t3088, t3089.
+ */
+int build_graph_group_cb(void *arg, struct tup_entry *tent)
+{
+	struct graph *g = arg;
+	struct node *n;
+
+	n = find_or_create_node(g, tent);
+	if(!n)
+		return -1;
+
+	if(n->expanded == 0) {
+		expand_node(g, n);
+	}
+
+	if(create_edge(g->cur, n, TUP_LINK_NORMAL) < 0)
+		return -1;
+
+	return 0;
+}
+
+/* Callback for adding group dependencies to the circular dependency check
+ * graph.
+ */
 static int build_group_cb(void *arg, struct tup_entry *tent,
 			  struct tup_entry *cmdtent)
 {
@@ -338,6 +364,10 @@ int build_graph(struct graph *g)
 			} else {
 				if(tup_db_select_node_by_link(build_graph_cb, g, cur->tnode.tupid) < 0)
 					return -1;
+				if(g->cur->tent->type == TUP_NODE_GROUP) {
+					if(tup_db_select_node_by_distinct_group_link(build_graph_group_cb, g, cur->tnode.tupid) < 0)
+						return -1;
+				}
 			}
 			cur->state = STATE_PROCESSING;
 		} else if(cur->state == STATE_PROCESSING) {
@@ -499,10 +529,10 @@ static void mark_nodes(struct node *n)
 		LIST_FOREACH(e2, &n->edges, list) {
 			struct node *dest = e2->dest;
 
-			/* Groups are skipped, otherwise we end up
-			 * building everything in the group (t3058).
+			/* Groups and exclusions are skipped, otherwise we end
+			 * up building everything in the group (t3058, t5099).
 			 */
-			if(dest->tent->type != TUP_NODE_GROUP) {
+			if(dest->tent->type != TUP_NODE_GROUP && dest->tent->dt != exclusion_dt()) {
 				mark_nodes(dest);
 			}
 		}
@@ -516,16 +546,9 @@ static void mark_nodes(struct node *n)
 	}
 }
 
-static int prune_node(struct graph *g, struct node *n, int *num_pruned, int verbose)
+static int prune_node(struct graph *g, struct node *n, int *num_pruned, enum graph_prune_type gpt, int verbose)
 {
 	if(n->tent->type == g->count_flags && n->expanded) {
-		g->num_nodes--;
-		if(g->total_mtime != -1) {
-			if(n->tent->mtime != -1)
-				g->total_mtime -= n->tent->mtime;
-		}
-		(*num_pruned)++;
-
 		if(n->tent->type != TUP_NODE_CMD) {
 			fprintf(stderr, "tup internal error: node of type %i trying to add to the modify list in prune_graph\n", n->tent->type);
 			return -1;
@@ -536,18 +559,31 @@ static int prune_node(struct graph *g, struct node *n, int *num_pruned, int verb
 		 */
 		if(tup_db_add_modify_list(n->tent->tnode.tupid) < 0)
 			return -1;
+
+		g->num_nodes--;
+		if(g->total_mtime != -1) {
+			if(n->tent->mtime != -1)
+				g->total_mtime -= n->tent->mtime;
+		}
+		(*num_pruned)++;
 		if(verbose) {
 			printf("Skipping: ");
 			print_tup_entry(stdout, n->tent);
 			printf("\n");
 		}
 	}
-	remove_node_internal(g, n);
+	/* Normal files are not pruned so we can make sure we update the mtime
+	 * in the tup database (t6079). Any dependent commands are flagged as
+	 * modify above, so they will still run later.
+	 */
+	if(gpt == GRAPH_PRUNE_ALL || n->tent->type != TUP_NODE_FILE) {
+		remove_node_internal(g, n);
+	}
 	return 0;
 }
 
 int prune_graph(struct graph *g, int argc, char **argv, int *num_pruned,
-		int verbose)
+		enum graph_prune_type gpt, int verbose)
 {
 	struct tup_entry_head *prune_list;
 	struct tupid_entries dir_root = {NULL};
@@ -624,7 +660,7 @@ int prune_graph(struct graph *g, int argc, char **argv, int *num_pruned,
 
 		TAILQ_FOREACH_SAFE(n, &g->node_list, list, tmp) {
 			if(!n->marked && n != g->root)
-				if(prune_node(g, n, num_pruned, verbose) < 0)
+				if(prune_node(g, n, num_pruned, gpt, verbose) < 0)
 					goto out_err;
 		}
 	}
@@ -667,7 +703,8 @@ void save_graph(FILE *err, struct graph *g, const char *filename)
 		perror("asprintf");
 		return;
 	}
-	fprintf(err, "tup: saving graph '%s'\n", realfile);
+	if(err)
+		fprintf(err, "tup: saving graph '%s'\n", realfile);
 
 	count++;
 	f = fopen(realfile, "w");
@@ -858,16 +895,6 @@ static tupid_t command_name_hash(tupid_t hash, struct tup_entry *tent,
 	return hash;
 }
 
-static void free_string_tree(struct string_entries *root)
-{
-	struct string_tree *st;
-	while(!RB_EMPTY(root)) {
-		st = RB_MIN(string_entries, root);
-		string_tree_free(root, st);
-		free(st);
-	}
-}
-
 static tupid_t command_outgoing_hash(tupid_t hash, struct node *n)
 {
 	struct edge *e;
